#! /usr/bin/env node

'use strict';

const Co = require('co'),
    Fs = require('fs'),
    Moment = require('moment'),
    OS = require('os'),
    Path = require('path'),
    inquirer = require('inquirer'),
    Prompt = require('prompt'),
    _ = require('lodash'),
    spawn = require('child_process').spawn,
    exec = require('child_process').exec;

// write file
function writeFile(modelFilePath, model) {
    return (done) => {
        Fs.writeFile(modelFilePath, model, 'utf8', done);
    };
}

const generateTemplate = Co.wrap(function* (type) {
    return function () {

    }
});


const SERVER_PATH = Path.resolve('server');
const ROUTES_PATH = Path.resolve(SERVER_PATH, 'routes');
const MODELS_PATH = Path.resolve(SERVER_PATH, 'models');
const CONTROLLERS_PATH = Path.resolve(SERVER_PATH, 'controllers');

const ROUTE_TEMPLATE = Fs.readFileSync(Path.resolve(__dirname, '../template/route.template'), 'utf8');
const MODEL_TEMPLATE = Fs.readFileSync(Path.resolve(__dirname, '../template/model.template'), 'utf8');
const CONTROLLER_GLOBAL_TEMPLATE = Fs.readFileSync(Path.resolve(__dirname, '../template/controller_global.template'), 'utf8');
const CONTROLLER_TEMPLATE = Fs.readFileSync(Path.resolve(__dirname, '../template/controller.template'), 'utf8');

const API_REPOSITORY = 'https://yeobster@bitbucket.org/yeobster/api_server.git';

let userArgs = process.argv.slice(2);
if (userArgs[0] === 'g') {
    if (userArgs[1]) {
        console.log('\x1b[32m# 프로젝트 생성 ' + userArgs[1] + '\x1b[0m\n');
        exec('git clone ' + API_REPOSITORY + ' ' + userArgs[1],
            (err, stdout, stderr) => {
                if (err) {
                    console.log('\x1b[31m#' + err + '\x1b[0m\n');
                    return;
                }
                if (stdout) {
                    console.log('\x1b[34m#' + stdout + '\x1b[0m\n');
                }
                console.log('프로젝트 생성됨 cd ' + userArgs[1] + ' && npm install');
            });
    } else {
        console.log('\x1b[31m# 프로젝트명을 입력하시오.\x1b[0m\n');
    }
} else if (userArgs[0] === 'a' || userArgs[0] === 'add') {
    // 추가
    console.log(SERVER_PATH);
    if (!Fs.existsSync(SERVER_PATH)) {
        console.log('\x1b[31m# 프로젝트의 root 디렉토리 에서 실행 하시오.\x1b[0m\n');
    } else if (!Fs.existsSync(ROUTES_PATH)) {
        console.log('\x1b[31m# routes 디렉토리가 존재하지 않습니다.\x1b[0m\n');
    } else if (!Fs.existsSync(MODELS_PATH)) {
        console.log('\x1b[31m# model 디렉토리가 존재하지 않습니다.\x1b[0m\n');
    } else if (!Fs.existsSync(CONTROLLERS_PATH)) {
        console.log('\x1b[31m# controller 디렉토리가 존재하지 않습니다.\x1b[0m\n');
    } else {
        if (userArgs[1]) {
            inquirer.prompt([
                {
                    type: 'list',
                    message: 'Generate template',
                    name: 'type',
                    choices: [
                        {
                            name: '[Exit]'
                        },
                        {
                            name: 'All'
                        },
                        {
                            name: 'Controller'
                        },
                        {
                            name: 'Model'
                        },
                        {
                            name: 'Route'
                        }
                    ],
                    validate: (result) => {
                        if (result.length < 1) {
                            return 'You must choose at least one template.';
                        }
                        return true;
                    }
                }
            ]).then((template) => {
                if (template.type === '[Exit]') {
                    // 종료
                    console.log('exit');
                    return;
                }
                // set template
                let entity = userArgs[1];
                let capEntity = _.capitalize(userArgs[1]);

                let modelFilePath = Path.resolve(MODELS_PATH, entity + '.js');
                let routeFilePath = Path.resolve(ROUTES_PATH, entity + '.js');
                let controllerFilePath = Path.resolve(CONTROLLERS_PATH, entity + '.js');

                let route = _.template(ROUTE_TEMPLATE)({
                    'entity': entity,
                    'username': OS.userInfo().username,
                    'createdAt': Moment().format('YYYY.MM.DD'),
                    'createdYear': Moment().format('YYYY')
                });
                let controller = _.template(CONTROLLER_GLOBAL_TEMPLATE)({
                    'entity': entity,
                    'cap_entity': capEntity,
                    'username': OS.userInfo().username,
                    'createdAt': Moment().format('YYYY.MM.DD'),
                    'createdYear': Moment().format('YYYY')
                });

                // check duplicate template
                if (template.type === 'All' || template.type === 'Model') {
                    // select db
                    const dbs = ['[Exit]', 'disk', 'memory', 'mongo', 'mysql', 'postgresql', 'redis', 'sqlserver'];
                    let description = '\n[Support Database]';
                    for (let index in dbs) {
                        description += '\n' + index + '. ' + dbs[index];
                    }
                    description += '\n Select Database (1-' + dbs.length + ')';
                    inquirer.prompt([
                        {
                            type: 'list',
                            name: 'db',
                            message: description,
                            choices: dbs,
                            validate: (result) => {
                                if (result.length < 1) {
                                    return 'You must choose at least one template.';
                                }
                                return true;
                            }
                        }
                    ]).then((select) => {
                        if (select.db === '[Exit]') {
                            console.log('exit');
                            return;
                        } else {
                            let selectDB = select.db;
                            console.log('\x1b[32m# Select ' + selectDB + '\x1b[0m\n');

                            let model = _.template(MODEL_TEMPLATE)({
                                'entity': entity, 'database': selectDB,
                                'username': OS.userInfo().username,
                                'createdAt': Moment().format('YYYY.MM.DD'),
                                'createdYear': Moment().format('YYYY')
                            });

                            Co(function* () {
                                try {
                                    console.log('template = ' + template.type);
                                    if (template.type === 'All') {
                                        if (Fs.existsSync(modelFilePath)) {
                                            console.log('\x1b[31m# Duplicate Model \x1b[0m\n');
                                            return false;
                                        } else {
                                            yield writeFile(modelFilePath, model, 'utf8');
                                            console.log('\x1b[32m>> create ' + modelFilePath + '\x1b[0m\n');
                                        }

                                        if (Fs.existsSync(routeFilePath)) {
                                            console.log('\x1b[31m# Duplicate Route \x1b[0m\n');
                                            return false;
                                        } else {
                                            yield writeFile(routeFilePath, route, 'utf8');
                                            console.log('\x1b[32m>> create ' + routeFilePath + '\x1b[0m\n');
                                        }

                                        if (Fs.existsSync(controllerFilePath)) {
                                            console.log('\x1b[31m# Duplicate Controller \x1b[0m\n');
                                            return false;
                                        } else {
                                            yield writeFile(controllerFilePath, controller, 'utf8');
                                            console.log('\x1b[32m>> create ' + controllerFilePath + '\x1b[0m\n');
                                        }
                                    } else if (template.type === 'Model') {
                                        if (Fs.existsSync(modelFilePath)) {
                                            console.log('\x1b[31m# Duplicate Model \x1b[0m\n');
                                            return false;
                                        } else {
                                            yield writeFile(modelFilePath, model, 'utf8');
                                            console.log('\x1b[32m>> create ' + modelFilePath + '\x1b[0m\n');
                                        }
                                    }
                                    return true;
                                } catch (err) {
                                    throw err;
                                }
                            }).then((created) => {
                                if (created) {
                                    console.log('\x1b[32m>> created \x1b[0m\n');
                                    console.log('\x1b[32m## node restart ##\x1b[0m\n');
                                } else {
                                    console.log('\x1b[32m>> done \x1b[0m\n');
                                }
                            }).catch((err) => {
                                console.log('\x1b[31m# 파일 작성 오류 \x1b[0m\n');
                            });
                        }
                    });
                } else {
                    Co(function* () {
                        try {
                            if (template.type === 'Route') {
                                if (Fs.existsSync(routeFilePath)) {
                                    console.log('\x1b[31m# Duplicate Route \x1b[0m\n');
                                    return false;
                                } else {
                                    yield writeFile(routeFilePath, route, 'utf8');
                                    console.log('\x1b[32m>> create ' + routeFilePath + '\x1b[0m\n');
                                }
                            } else if (template.type === 'Controller') {
                                if (Fs.existsSync(controllerFilePath)) {
                                    console.log('\x1b[31m# Duplicate Controller \x1b[0m\n');
                                    return false;
                                } else {
                                    yield writeFile(controllerFilePath, controller, 'utf8');
                                    console.log('\x1b[32m>> create ' + controllerFilePath + '\x1b[0m\n');
                                }
                            }
                            return true;
                        } catch (err) {
                            throw err;
                        }
                    }).then((created) => {
                        if (created) {
                            console.log('\x1b[32m>> created \x1b[0m\n');
                            console.log('\x1b[32m## node restart ##\x1b[0m\n');
                        } else {
                            console.log('\x1b[32m>> done \x1b[0m\n');
                        }
                    }).catch((err) => {
                        console.error(err);
                        console.log('\x1b[31m# 파일 작성 오류 \x1b[0m\n');
                    });
                }
            });
        } else {
            console.log('\x1b[31m# 모델명을 입력하세요.\x1b[0m\n');
        }
    }
} else if (userArgs[0] === 'r') {
    // 삭제
    if (userArgs[1]) {

        inquirer.prompt([
            {
                type: 'confirm',
                name: 'delete',
                message: '삭제된 모델은 복구가 불가능 합니다.',
                default: false
            }
        ]).then((result) => {
            if (result.delete) {
                let entity = userArgs[1];
                let modelFilePath = Path.resolve(MODELS_PATH, entity + '.js');
                let routeFilePath = Path.resolve(ROUTES_PATH, entity + '.js');
                let controllerFilePath = Path.resolve(CONTROLLERS_PATH, entity + '.js');

                if (Fs.existsSync(modelFilePath)) {
                    Fs.unlinkSync(modelFilePath);
                    console.log('\x1b[32m>> remove ' + modelFilePath + '\x1b[0m');
                }
                if (Fs.existsSync(routeFilePath)) {
                    Fs.unlinkSync(routeFilePath);
                    console.log('\x1b[32m>> remove ' + modelFilePath + '\x1b[0m');
                }
                if (Fs.existsSync(controllerFilePath)) {
                    Fs.unlinkSync(controllerFilePath);
                    console.log('\x1b[32m>> remove ' + modelFilePath + '\x1b[0m');
                }
            } else {
                console.log('취소 했습니다.');
            }
        });
    } else {
        console.log('\x1b[31m# 모델명을 입력하세요.\x1b[0m\n');
    }
} else if (userArgs[0] === 'v') {
    // 패키지 버전
    let packageJson = require(Path.resolve(__dirname, '../package.json'));
    console.log('version ' + packageJson.version);

} else if (userArgs[0] === 'h' || !userArgs[0]) {
    // 도움말
    console.log('\n');
    console.log(' 사용법: smds [명령어] [옵션(모델명)]');
    console.log('\n');
    console.log(' 명령어:');
    console.log('          add [옵션(모델명)] : 새 모델 추가 ex) smds add users');
    console.log('          a  [옵션(모델명)] : 새 모델 추가 ex) smds a users');
    console.log('          r  [옵션(모델명)] : 모델 삭제 ex) smds r users');
    console.log('          -v : samdasoo 버전');
    console.log('          -h : samdasoo 도움말');
    console.log('\n');
}